/*
 * control_ipc.c
 */

/*
 * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
 */

/* 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
 
/************************************************************************
*  This file implements receiving remote procedure calls from the GPP   *
*  and calling the appropriate local DSP function and returning results *
************************************************************************/

/*  ----------------------------------- Standard headers            */
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>  
#include <stdbool.h>

/*  ----------------------------------- DSP/BIOS Headers            */
#include "C6Runcfg.h"
#include <gbl.h>
#include <sys.h>
#include <sem.h>
#include <msgq.h>
#include <pool.h>
#include <tsk.h>
#include <log.h>

/*  ----------------------------------- DSP/BIOS LINK Headers       */
#include <dsplink.h>
#include <platform.h>
#include <failure.h>
#if defined (MSGQ_ZCPY_LINK)
  #include <zcpy_mqt.h>
  #include <sma_pool.h>
#endif

/*  ----------------------------------- This module's header        */
#include "rpc_ipc.h"


/************************************************************
* Explicit External Declarations                            *
************************************************************/


/************************************************************
* Local Macro Declarations                                  *
************************************************************/


/************************************************************
* Local Typedef Declarations                                *
************************************************************/


/************************************************************
* Local Function Declarations                               *
************************************************************/


/************************************************************
* Local Variable Definitions                                *
************************************************************/

static SEM_Obj RPC_IPC_NotifySemObj;
static bool LOCAL_isModuleInitialized = false;


/************************************************************
* Global Variable Definitions                               *
************************************************************/

MSGQ_Queue MSGQ_rpcFromGPP, MSGQ_rpcToGPP;
RPC_Msg *RPC_msg;


/************************************************************
* Global Function Definitions                               *
************************************************************/

int32_t RPC_IPC_init( void )
{
  int32_t status = SYS_OK;
  MSGQ_Attrs MSGQ_attrs = MSGQ_ATTRS;
  MSGQ_LocateAttrs MSGQ_locateAttrs;
  
  // If already initialized, just exit
  if (LOCAL_isModuleInitialized)
  {
    return status;
  }
  
  // Init the message queues
  MSGQ_rpcFromGPP       = MSGQ_INVALIDMSGQ; 
  MSGQ_rpcToGPP         = MSGQ_INVALIDMSGQ;
  
  // Init semaphore objects that underly the message queues
  SEM_new(&RPC_IPC_NotifySemObj, 0);

  // Fill in the attributes for this message queue
  MSGQ_attrs.notifyHandle = &RPC_IPC_NotifySemObj;
  MSGQ_attrs.pend = (MSGQ_Pend) SEM_pendBinary;
  MSGQ_attrs.post = (MSGQ_Post) SEM_postBinary;
  
  // Open Message Queues
  status = MSGQ_open((String) RPC_FROM_GPP_MSGQNAME, &MSGQ_rpcFromGPP, &MSGQ_attrs);
  if (status != SYS_OK)
  {
    LOG_error("MSGQ_open failed", 0);
    return status;
  }
  
  // Locate the GPP-side receive message queues
  MSGQ_locateAttrs.timeout = SYS_FOREVER;
  while(1)
  {
    status = MSGQ_locate((String) RPC_TO_GPP_MSGQNAME, &MSGQ_rpcToGPP, &MSGQ_locateAttrs);
    if (status == SYS_OK)
    {
      break;
    }

    if (status == SYS_ENOTFOUND || status == SYS_ENODEV)
    {
      TSK_sleep(1000);
      continue;
    }

    LOG_error("MSGQ_locate failed", 0);
    break;
  }
  
  LOCAL_isModuleInitialized = true;
  
  return status;
}

void RPC_IPC_exit( void )
{
  int32_t status;
  
  // If not already initialized, just exit
  if (!LOCAL_isModuleInitialized)
  {
    return;
  }  
  
  // Release the CIO to-GPP message queue
  if (MSGQ_rpcToGPP != MSGQ_INVALIDMSGQ)
  {
    status = MSGQ_release(MSGQ_rpcToGPP);
    if (status != SYS_OK)
    {
      LOG_error("MSGQ_release failed", 0);
    }
  }

  // Delete the local CIO from-GPP message queue
  if (MSGQ_rpcFromGPP != MSGQ_INVALIDMSGQ)
  {
    status = MSGQ_close(MSGQ_rpcFromGPP);
    if (status != SYS_OK)
    {
      LOG_error("MSGQ_close failed", 0);
    }
  }
  LOCAL_isModuleInitialized = false;
}

// RPC_IPC_sendMsg - Sends the data and parameters on to the GPP
void RPC_IPC_sendMsg( RPC_Msg *msg, uint16_t msgId )
{
  Int status;

  // Setup message and send  
  MSGQ_setMsgId((MSGQ_Msg) msg, msgId);
  MSGQ_setSrcQueue((MSGQ_Msg) msg, MSGQ_rpcFromGPP);
  
  status = MSGQ_put(MSGQ_rpcToGPP, (MSGQ_Msg) msg);
  if (status != SYS_OK)
  {
    LOG_error("MSGQ_put failed", 0);
    MSGQ_free((MSGQ_Msg) msg);
  }
}

// RPC_IPC_recvMsg - Reads the data and parameters passed from the GPP
uint16_t RPC_IPC_recvMsg( RPC_Msg **msg )
{
  Int status;

  status = MSGQ_get(MSGQ_rpcFromGPP, (MSGQ_Msg *) msg, SYS_FOREVER);
  if (status != SYS_OK)
  {
    LOG_error("MSGQ_get failed", 0);
    return RPC_NULL;
  }
  
  return MSGQ_getMsgId((MSGQ_Msg) (*msg));
}


/***********************************************************
* Local Function Definitions                               *
***********************************************************/


/***********************************************************
* End file                                                 *
***********************************************************/
