/*
 * rpc_ipc.c
 */

/*
 * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
 */

/* 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/************************************************************************
*  Routines for handling C I/O requests from the DSP                    *
************************************************************************/

/*  ----------------------------------- Standard Headers                */
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <time.h>

/*  ----------------------------------- Linux Posix Headers            */
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <unistd.h>
#include <semaphore.h>

/*  ----------------------------------- DSP/BIOS Link                   */
#include <dsplink.h>

/*  ----------------------------------- DSP/BIOS LINK API               */
#include <proc.h>
#include <procdefs.h>
#include <msgq.h>
#include <pool.h>
#include <loaderdefs.h>

/*  ----------------------------------- Application headers             */
#include "debug_common.h"
#include "C6Run.h"
#include "C6Run_mem.h"

/*  ----------------------------------- This module's header            */
#include "rpc_ipc.h"


/************************************************************
* Explicit External Declarations                            *
************************************************************/


/************************************************************
* Local Macro Declarations                                  *
************************************************************/


/************************************************************
* Local Typedef Declarations                                *
************************************************************/


/************************************************************
* Local Function Declarations                               *
************************************************************/


/************************************************************
* Local Variable Definitions                                *
************************************************************/

static sem_t RPC_IPC_msgSemObj;

static bool LOCAL_isModuleInitialized = false;


/************************************************************
* Global Variable Definitions                               *
************************************************************/

// Global CONTROL MSGQ Objects for Tx and Rx
MSGQ_Queue MSGQ_rpcFromGPP = (uint32_t) MSGQ_INVALIDMSGQ;
MSGQ_Queue MSGQ_rpcToGPP = (uint32_t) MSGQ_INVALIDMSGQ;

RPC_Msg *RPC_msg;


/************************************************************
* Global Function Definitions                               *
************************************************************/

int RPC_IPC_init()
{
  DSP_STATUS status = DSP_SOK;

  if (LOCAL_isModuleInitialized)
  {
    return C6RUN_SUCCESS;
  }
  
  VERBOSE_PRINT0("Entered RPC_IPC_init()\n");

  // Open the RPC TO GPP message queue
  if (DSP_SUCCEEDED(status))
  {
    status = MSGQ_open(RPC_TO_GPP_MSGQNAME, &MSGQ_rpcToGPP, NULL);
    if (DSP_FAILED(status))
    {
      REPORT_STATUS("MSGQ_open");
      return C6RUN_FAIL;
    }
  }
  
  // Locate the RPC FROM GPP message queue
  if (DSP_SUCCEEDED(status))
  {
    MSGQ_LocateAttrs syncLocateAttrs;
  
    syncLocateAttrs.timeout = WAIT_FOREVER;
    status = DSP_ENOTFOUND;
    while ((status == DSP_ENOTFOUND) || (status == DSP_ENOTREADY))
    {
      status = MSGQ_locate(RPC_FROM_GPP_MSGQNAME, &MSGQ_rpcFromGPP, &syncLocateAttrs);
      if ((status == DSP_ENOTFOUND) || (status == DSP_ENOTREADY))
      {
        VERBOSE_PRINT0("RPC MSGQ_locate() failed! Will retry.\n");
        usleep(10000);
      }
      else if (DSP_FAILED(status))
      {
        REPORT_STATUS("MSGQ_locate");
        return C6RUN_FAIL;
      }
    }
  }
  
  if( sem_init(&RPC_IPC_msgSemObj,0,RPC_MSG_CNT) < 0 )
  {
    printf("sem_init() failed");
    exit(0);
  }

  
  LOCAL_isModuleInitialized = true;
  
  VERBOSE_PRINT0("Leaving RPC_IPC_init()\n");
  
  return C6RUN_SUCCESS;
}

void RPC_IPC_exit()
{
  DSP_STATUS status = DSP_SOK;
  DSP_STATUS tmpStatus = DSP_SOK;
  
  if (!LOCAL_isModuleInitialized)
  {
    return;
  }

  VERBOSE_PRINT0("Entered RPC_IPC_exit()\n");
  
  // Release the remote message queue
  status = MSGQ_release(MSGQ_rpcFromGPP);
  if (DSP_FAILED(status))
  {
    REPORT_STATUS("MSGQ_release");
  }
  else
  {
    VERBOSE_PRINT0("MSGQ_release() passed.\n");
  }

  // Close the GPP's RPC message queue
  tmpStatus = MSGQ_close(MSGQ_rpcToGPP);
  if (DSP_SUCCEEDED(status) && DSP_FAILED(tmpStatus))
  {
    status = tmpStatus;
    REPORT_STATUS("MSGQ_close");
  }
  else
  {
    VERBOSE_PRINT0("MSGQ_close() passed.\n");
  }

  LOCAL_isModuleInitialized = false;

  VERBOSE_PRINT0("Leaving RPC_IPC_exit()\n");
}

void RPC_IPC_sendMsg( RPC_Msg *msg, uint16_t msgId )
{
  DSP_STATUS status;
  
  VERBOSE_PRINT0("Entering RPC_IPC_sendMsg()\n");

  MSGQ_setMsgId((MsgqMsg)msg, msgId);

  status = MSGQ_put(MSGQ_rpcFromGPP, (MsgqMsg) msg);
  
  if (DSP_FAILED(status))
  { 
    MSGQ_free((MsgqMsg) msg); 
    REPORT_STATUS("MSGQ_put"); 
  }
  
  VERBOSE_PRINT0("Exiting RPC_IPC_sendMsg()\n");
}

uint16_t RPC_IPC_recvMsg( RPC_Msg **msg)
{
  DSP_STATUS status;
  uint16_t msgId;
  
  VERBOSE_PRINT0("Entering RPC_IPC_recvMsg()\n");

  status = MSGQ_get(MSGQ_rpcToGPP, WAIT_FOREVER, (MsgqMsg *) msg);
  if (DSP_FAILED(status))
  {
    REPORT_STATUS("MSGQ_get"); 
    return RPC_NULL;
  }
  msgId = MSGQ_getMsgId((MsgqMsg) (*msg));

  VERBOSE_PRINT0("Exiting RPC_IPC_recvMsg()\n");
  
  return msgId;
}

void RPC_IPC_allocMsg( RPC_Msg **msg )
{

  VERBOSE_PRINT0("Entering RPC_IPC_allocMsg()\n");

  // Semaphore wait on an available message
  sem_wait(&RPC_IPC_msgSemObj);
  
  {
    DSP_STATUS status = DSP_SOK;
    
    // Do MSGQ alloc for a remote procedure call
    status = MSGQ_alloc(COMMON_POOL_ID, sizeof(RPC_Msg), (MSGQ_Msg *) msg );
    if (DSP_FAILED(status))
    { 
      REPORT_STATUS("MSGQ_alloc");
      *msg =  NULL;
    }
  }
  
  VERBOSE_PRINT0("Exiting RPC_IPC_allocMsg()\n");
}

void RPC_IPC_freeMsg( RPC_Msg *msg )
{

  VERBOSE_PRINT0("Entering RPC_IPC_freeMsg()\n");

  MSGQ_free((MSGQ_Msg) msg);
  sem_post(&RPC_IPC_msgSemObj);
  
  VERBOSE_PRINT0("Exiting RPC_IPC_freeMsg()\n");
}


/***********************************************************
* Local Function Definitions                               *
***********************************************************/


/***********************************************************
* End file                                                 *
***********************************************************/

